package core

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
)

// CompileIf compiles a branch instruction.
func (f *Function) CompileIf(branch *ast.If) error {
	condition := branch.Condition
	tmp := f.cpu.MustFindFree(f.cpu.General)
	err := f.ExpressionToRegister(condition.Children[0], tmp)

	if err != nil {
		return err
	}

	f.cpu.Use(tmp)
	err = f.Execute(condition.Token, tmp, condition.Children[1])

	if err != nil {
		return err
	}

	f.cpu.Free(tmp)
	endLabel := fmt.Sprintf("%s_end_if_%d", f.Name, f.count.branch)

	switch condition.Token.Text() {
	case "==":
		f.assembler.Label(asm.JNE, endLabel)
	case "!=":
		f.assembler.Label(asm.JE, endLabel)
	case ">":
		f.assembler.Label(asm.JLE, endLabel)
	case "<":
		f.assembler.Label(asm.JGE, endLabel)
	case ">=":
		f.assembler.Label(asm.JL, endLabel)
	case "<=":
		f.assembler.Label(asm.JG, endLabel)
	}

	defer f.assembler.Label(asm.LABEL, endLabel)
	f.count.branch++
	return f.CompileAST(branch.Body)
}
